<template>
  <div class="tinymce-box">
    <Editor
      v-model="contentValue"
      :init="init"
      :disabled="disabled"
      @onClick="onClick"
      :key="tinymceFlag"
    />
  </div>
</template>

<script>
//引入tinymce编辑器
import Editor from '@tinymce/tinymce-vue'

//引入node_modules里的tinymce相关文件文件
import tinymce from 'tinymce/tinymce' //tinymce默认hidden，不引入则不显示编辑器
import 'tinymce/themes/silver' //编辑器主题，不引入则报错
//import 'tinymce/icons/default'  //引入编辑器图标icon，不引入则不显示对应图标

// 引入编辑器插件（基本免费插件都在这儿了）
import 'tinymce/plugins/advlist' //高级列表
import 'tinymce/plugins/anchor' //锚点
import 'tinymce/plugins/autolink' //自动链接
import 'tinymce/plugins/autoresize' //编辑器高度自适应,注：plugins里引入此插件时，Init里设置的height将失效
import 'tinymce/plugins/autosave' //自动存稿
import 'tinymce/plugins/charmap' //特殊字符
import 'tinymce/plugins/code' //编辑源码
import 'tinymce/plugins/codesample' //代码示例
import 'tinymce/plugins/directionality' //文字方向
import 'tinymce/plugins/emoticons' //表情
import 'tinymce/plugins/fullpage' //文档属性
import 'tinymce/plugins/fullscreen' //全屏
import 'tinymce/plugins/help' //帮助
import 'tinymce/plugins/hr' //水平分割线
import 'tinymce/plugins/image' //插入编辑图片
import 'tinymce/plugins/importcss' //引入css
import 'tinymce/plugins/insertdatetime' //插入日期时间
import 'tinymce/plugins/link' //超链接
import 'tinymce/plugins/lists' //列表插件
import 'tinymce/plugins/media' //插入编辑媒体
import 'tinymce/plugins/nonbreaking' //插入不间断空格
import 'tinymce/plugins/pagebreak' //插入分页符
import 'tinymce/plugins/paste' //粘贴插件
import 'tinymce/plugins/preview' //预览
import 'tinymce/plugins/print' //打印
import 'tinymce/plugins/quickbars' //快速工具栏
import 'tinymce/plugins/save' //保存
import 'tinymce/plugins/searchreplace' //查找替换
// import 'tinymce/plugins/spellchecker'  //拼写检查，暂未加入汉化，不建议使用
import 'tinymce/plugins/tabfocus' //切入切出，按tab键切出编辑器，切入页面其他输入框中
import 'tinymce/plugins/table' //表格
import 'tinymce/plugins/template' //内容模板
import 'tinymce/plugins/textcolor' //文字颜色
import 'tinymce/plugins/textpattern' //快速排版
import 'tinymce/plugins/toc' //目录生成器
import 'tinymce/plugins/visualblocks' //显示元素范围
import 'tinymce/plugins/visualchars' //显示不可见字符
import 'tinymce/plugins/wordcount' //字数统计
import {mapState} from 'vuex'
import axios from 'axios'
export default {
  name: 'TEditor',
  components: {
    Editor
  },
  props: {
    value: {
      type: String,
      default: ''
    },
    disabled: {
      type: Boolean,
      default: false
    },
    plugins: {
      type: [String, Array],
      default:
        'print preview searchreplace autolink directionality visualblocks visualchars fullscreen image link media template code codesample table charmap hr pagebreak nonbreaking anchor insertdatetime advlist lists wordcount textpattern autosave '
    },
    toolbar: {
      type: [String, Array],
      default:
        'fullscreen undo redo restoredraft | cut copy paste pastetext | forecolor backcolor bold italic underline strikethrough link anchor | alignleft aligncenter alignright alignjustify outdent indent | image link media|\
                styleselect formatselect fontselect fontsizeselect | bullist numlist | blockquote subscript superscript removeformat | \
                table image charmap hr pagebreak insertdatetime print preview | code selectall searchreplace visualblocks | indent2em lineheight formatpainter axupimgs'
    }
  },
  data() {
    return {
      tinymceFlag: 1,
      init: {
        media_url_resolver: function(data, resolve) {
          console.log('data:', data)
          console.log('resolve:', resolve)
          try {
            let videoUri = encodeURI(data.url)
            let embedHtml = `<p>
                            <span
                                class="mce-object mce-object-video"
                                data-mce-selected="1"
                                data-mce-object="video"
                                data-mce-p-width="100%"
                                data-mce-p-height="auto"
                                data-mce-p-controls="controls"
                                data-mce-p-controlslist="nodownload"
                                data-mce-p-allowfullscreen="true"
                                data-mce-p-src=${videoUri} >
                                <video src=${data.url} width="100%" height="auto" controls="controls" controlslist="nodownload">
                                </video>
                            </span>
                        </p>
                        <p style="text-align: left;"></p>`
            resolve({html: embedHtml})
          } catch (e) {
            resolve({html: ''})
          }
        },
        language_url: '/tinymce/langs/zh_CN.js', //引入语言包文件
        language: 'zh_CN', //语言类型

        skin_url: '/tinymce/skins/ui/oxide', //皮肤：浅色
        // skin_url: '/tinymce/skins/ui/oxide-dark',//皮肤：暗色

        plugins: this.plugins, //插件配置
        toolbar: this.toolbar, //工具栏配置，设为false则隐藏
        // menubar: 'file edit',  //菜单栏配置，设为false则隐藏，不配置则默认显示全部菜单，也可自定义配置--查看 http://tinymce.ax-z.cn/configure/editor-appearance.php --搜索“自定义菜单”

        fontsize_formats:
          '12px 14px 16px 18px 20px 22px 24px 28px 32px 36px 48px 56px 72px', //字体大小
        font_formats:
          '微软雅黑=Microsoft YaHei,Helvetica Neue,PingFang SC,sans-serif;苹果苹方=PingFang SC,Microsoft YaHei,sans-serif;宋体=simsun,serif;仿宋体=FangSong,serif;黑体=SimHei,sans-serif;Arial=arial,helvetica,sans-serif;Arial Black=arial black,avant garde;Book Antiqua=book antiqua,palatino;', //字体样式
        lineheight_formats: '0.5 0.8 1 1.2 1.5 1.75 2 2.5 3 4 5', //行高配置，也可配置成"12px 14px 16px 20px"这种形式

        height: 300, //注：引入autoresize插件时，此属性失效
        placeholder: '请输入公告内容',
        branding: false, //tiny技术支持信息是否显示
        resize: false, //编辑器宽高是否可变，false-否,true-高可变，'both'-宽高均可，注意引号
        // statusbar: false,  //最下方的元素路径和字数统计那一栏是否显示
        elementpath: false, //元素路径是否显示
        content_style: 'img {max-width:100%;}', //直接自定义可编辑区域的css样式
        paste_data_images: false, //图片是否可粘贴
        images_upload_handler: (blobInfo, success, failure) => {
          // 在这校验上传图片大小
          this.handleImgUpload(blobInfo, success, failure)
        },
        // 用于上传文件
        file_picker_callback: function(callback, value, meta) {
          let currentUser = JSON.parse(sessionStorage.getItem('currentUser'))
          //文件分类
          var filetype =
            '.pdf, .txt, .zip, .rar, .7z, .doc, .docx, .xls, .xlsx, .ppt, .pptx, .mp3, .mp4'
          //后端接收上传文件的路径
          var dir = ''
          //为不同插件指定文件类型及后端地址
          switch (meta.filetype) {
            case 'image':
              filetype = '.jpg, .jpeg, .png, .gif'
              dir = 'img'
              break
            case 'media':
              filetype = '.mp3, .mp4'
              dir = 'media'
              break
            case 'file':
            default:
          }
          //模拟出一个input用于添加本地文件
          let input = document.createElement('input')
          input.setAttribute('type', 'file')
          input.setAttribute('accept', filetype)
          input.click()
          input.onchange = function() {
            let file = this.files[0]
            let formData = new FormData()
            console.log(file)
            formData.append('file', file, file.name)
            axios({
              method: 'post', // post方法
              url: window.context.portal + '/biz/universal/v1/minioUpload', // 请求上传图片服务器的路径
              headers: {
                // 配置headers请求头
                Authorization: `Bearer ${currentUser.token}`, // token认证，看后台
                'Content-Type': 'application/x-www-form-urlencoded' // 采用表单上传的方式，看后台如何接受
              },
              data: formData // 请求数据formdata
            }).then(res => {
              console.log(res, 'ress')
              if (res.data.state != true) {
                // 上传失败执行此方法，将失败信息填入参数中
                return res.data.message
              }
              // 上传成功之后，将对应完整的图片路径拼接在success的参数中
              callback(res.data.value.url, {text: file.name})
            })
          }
        }
      },
      contentValue: this.value
    }
  },
  computed: {
    ...mapState({
      headers: state => {
        return {Authorization: `Bearer ${state.login.currentUser.token}`}
      },
      action: function() {
        return window.context.portal + '/biz/universal/v1/minioUpload'
      },
      uploadDisabled: function() {
        return this.fileList && this.fileList.length > 0
      }
    })
  },
  watch: {
    value(newValue) {
      this.contentValue = newValue
    },
    contentValue(newValue) {
      this.$emit('input', newValue)
    }
  },
  activated() {
    // 每次都给编辑器改变不同的key值使其每次切换页面都重新加载,防止无限递增
    this.tinymceFlag = this.tinymceFlag === 0 ? 1 : 0
  },
  mounted() {
    tinymce.init({})
  },
  methods: {
    // 添加相关的事件，可用的事件参照文档=> https://github.com/tinymce/tinymce-vue => All available events
    onClick(e) {
      this.$emit('onClick', e, tinymce)
    },
    //清空内容
    clear() {
      this.contentValue = ''
    },
    // 上传本地 图片执行事件
    handleImgUpload(blobInfo, success, failure) {
      let currentUser = JSON.parse(sessionStorage.getItem('currentUser'))
      let formdata = new FormData()
      // append 方法中的第一个参数就是 我们要上传文件 在后台接收的文件名
      // 这个值要根据后台来定义
      // 第二个参数是我们上传的文件
      formdata.append('file', blobInfo.blob())
      // axios 定义上传方法
      axios({
        method: 'post', // post方法
        url: window.context.portal + '/biz/universal/v1/minioUpload', // 请求上传图片服务器的路径
        headers: {
          // 配置headers请求头
          Authorization: `Bearer ${currentUser.token}`, // token认证，看后台
          'Content-Type': 'application/x-www-form-urlencoded' // 采用表单上传的方式，看后台如何接受
        },
        data: formdata // 请求数据formdata
      }).then(res => {
        console.log(res, 'ress')
        if (res.data.state != true) {
          // 上传失败执行此方法，将失败信息填入参数中
          failure(res.data.message)
          return
        }
        // 上传成功之后，将对应完整的图片路径拼接在success的参数中
        success(res.data.value.url)
      })
    }
  }
}
</script>
